import 'dart:async';
import 'dart:io';

import 'package:flutter/services.dart';

class GreedyGame {
  static const GG_INIT = "gg_init";
  static const GAME_ID = "game_id";
  static const UNITS = "units";

  static const CAMPAIGN_AVAILABLE = "gg_campaign_available";
  static const CAMPAIGN_UNAVAILABLE = "gg_campaign_unavailable";
  static const CAMPAIGN_ERROR = "gg_campaign_error";

  Function onCampaignAvailable;
  Function onCampaignUnAvailable;
  Function onCampaignError;

  GreedyGame.private(MethodChannel channel) : _channel = channel {
    _channel.setMethodCallHandler(_handleMethod);
  }

  final MethodChannel _channel;

  static final GreedyGame _instance =
      GreedyGame.private(const MethodChannel('flutter_greedygame_plugin'));

  Future<dynamic> _handleMethod(MethodCall call) {
    switch (call.method) {
      case CAMPAIGN_AVAILABLE:
        print("Campaign available");
        onCampaignAvailable(call.arguments);
        break;
      case CAMPAIGN_UNAVAILABLE:
        print("Campaign unavailable");
        onCampaignUnAvailable();
        break;
      case CAMPAIGN_ERROR:
        print("Campaign error");
        onCampaignError();
        break;
    }
    return Future<dynamic>.value(null);
  }

  /// The single shared instance of this plugin.
  static GreedyGame get instance => _instance;

  void init(String gameID, List<String> units, Function campaignAvailable,
      Function campaignUnAvailable, Function campaignError) {
    print("Initializing");
    onCampaignAvailable = campaignAvailable;
    onCampaignUnAvailable = campaignUnAvailable;
    onCampaignError = campaignError;

    Map<String, dynamic> args = <String, dynamic>{
      GAME_ID: gameID,
      UNITS: units,
    };
    _channel.invokeMethod(GG_INIT, args);
  }
}
